﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net.Sockets;
using System.Threading;
using System.Net;

namespace EtherIOProTCP
{

   
    
    public partial class Form1 : Form
    {
      



        int _EEReadDelay;
        bool _ReadingEE = false;

        UdpClient _udp;

        IPAddress _deviceIP;
        int _devicePort;
        IPEndPoint _deviceEP;

        private bool _scanned;




        
        public Form1()
        {
            InitializeComponent();

            _EEReadDelay = 10;
//            _scanned = false;
            _devicePort = 2424;
            _udp = new UdpClient(AddressFamily.InterNetwork);   // uses random outgoing port

        }

        private void TestTextBoxUDP_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            TestTextBoxUDP.Clear();
        }

        private void Form1_Activated(object sender, EventArgs e)
        {
            if (!_scanned)
            {
                _scanned = true;
                Module_Scan(null, null);
            }
        }

        /// <summary>
        /// scan for IO24 modules
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Module_Scan(object sender, EventArgs e)
        {
            tsbSearch.Enabled = false;
            btnGPIOTestUDP.Enabled = false;
            cbGPIOPortUDP.Enabled = false;
            TestTextBoxUDP.Enabled = false;


            // remove any existing drop-down items
            tsddModules.DropDownItems.Clear();
            tsddModules.Text = "No modules found";
            enableModuleItems(false);

            // broadcast "who's there"
            _udp.EnableBroadcast = true;
            _deviceIP = IPAddress.Parse("255.255.255.255");
            IPEndPoint ep = new IPEndPoint(_deviceIP, _devicePort);
            byte[] dgram = Encoding.UTF8.GetBytes("IO24");
            try
            {
                // send query packet
                _udp.Send(dgram, dgram.Length, ep);

                // wait for data
                waitForData(1000);
            }
            catch { }

            tsbSearch.Enabled = true;
        }

        /// <summary>
        /// set enabled status of Read/Write/Reboot options
        /// </summary>
        /// <param name="enabled"></param>
        private void enableModuleItems(bool enabled)
        {
            btnGPIOTestUDP.Enabled = true;
            cbGPIOPortUDP.Enabled = true;
            TestTextBoxUDP.Enabled = true;
            
           
        }

        /// <summary>
        /// wait until data received or timeout has occured
        /// </summary>
        /// <param name="timeout"></param>
        private bool waitForData(double timeout)
        {
            tmrCheckAvailable.Stop();

            if (_udp == null)
                return false;

            bool rslt = false;

            DateTime now = DateTime.Now;
            while (!rslt && DateTime.Now < now.AddMilliseconds(timeout))
            {
                if (_udp != null && _udp.Available >= 4)
                {
                    processUDPdata();
                    rslt = true;
                }
            }

            tmrCheckAvailable.Start();

            return rslt;
        }

        private void tmrCheckAvailable_Tick(object sender, EventArgs e)
        {
            if (_udp == null)
                return;
            else
                waitForData(10);
        }


        /// <summary>
        /// process any UDP data waiting
        /// </summary>
        private void processUDPdata()
        {
            if (_udp != null)
            {
                // loop while there is data to collect
                while (_udp.Available > 0)
                {
                    IPEndPoint ep = new IPEndPoint(IPAddress.Parse("255.255.255.255"), 0);
                    byte[] data = _udp.Receive(ref ep);
                    string ipAddress = ep.Address.ToString();
                    string text = Encoding.UTF8.GetString(data);

                    if (data.Length < 4)
                    {
                        _EEReadDelay += 10;
//                        debug.WriteLine("Short packet from {0}\r\nEEReadDelay increased to {1}\r\n", ipAddress, _EEReadDelay);
                    }
                    // check if this is an IO24 announcement
                    else if (text.Length > 4 && text.Substring(0, 4) == "IO24")
                    {
                        // only display information from modules
                        if (data.Length > 4)
                        {
                            if (!_ReadingEE)
                            {
                                // retrieve module information
                                string id = string.Format("{0} - MAC {1}:{2}:{3}:{4}:{5}:{6} V{7}.{8}",
                                    ipAddress,
                                    data[4].ToString("X2"), data[5].ToString("X2"), data[6].ToString("X2"), data[7].ToString("X2"), data[8].ToString("X2"), data[9].ToString("X2"),
                                    data[10], data[11]);

                                // add to list
                                ToolStripDropDownItem itm = new ToolStripMenuItem(id, null, new EventHandler(Module_Select));
                                itm.Tag = ep.Address.ToString();
                                tsddModules.DropDownItems.Add(itm);
                                tsddModules.Text = string.Format("{0} modules found", tsddModules.DropDownItems.Count);
                            }
                        }

                        // pause to allow time for other modules to respond
                        if (_udp.Available == 0)
                            System.Threading.Thread.Sleep(100);
                    }
                
                }
            }
        }


        /// <summary>
        /// set target device as selected module
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Module_Select(object sender, EventArgs e)
        {
            // update toolbar
            ToolStripMenuItem itm = (ToolStripMenuItem)sender;
            tsddModules.Text = itm.Text;
            enableModuleItems(true);

            // capture IP Address
            lDeviceIP.Text = (string)itm.Tag;
            _deviceIP = IPAddress.Parse(lDeviceIP.Text);
            _deviceEP = new IPEndPoint(_deviceIP, _devicePort);
            Application.DoEvents();
        }


      
        private void btnGPIOTestUDP_Click(object sender, EventArgs e)
        {
            UdpClient udpClient = new UdpClient();
            IPEndPoint EtherIPEndPoint = new IPEndPoint(_deviceIP, _devicePort);

            byte[] outBuffer = new byte[16];
            byte[] inBuffer = new byte[16];
            //byte inBufferCount = 0;

            byte port = (byte)'A';

            if (cbGPIOPortUDP.Text == "Port A")
                port = (byte)'A';
            else if (cbGPIOPortUDP.Text == "Port B")
                port = (byte)'B';
            else if (cbGPIOPortUDP.Text == "Port C")
                port = (byte)'C';

            outBuffer[0] = (byte)'!';
            outBuffer[1] = (byte)port;
            outBuffer[2] = (byte)0x64;

            udpClient.Send(outBuffer, 3, EtherIPEndPoint);
           
            
            //Initialise Port Values for GPIO Board
            // Set up SPI pins, CS and INT pins
            outBuffer[0] = (byte)port;
            outBuffer[1] = (byte)0x99;//Enable the CS_n for GPIO to idle state (1001 1001)
            udpClient.Send(outBuffer, 2, EtherIPEndPoint);

           
            Application.DoEvents();

            // Lower CS U1 = 0x91
            // Idle CS  = 0x99
            // U1 Control Byte = 0x42             // 0 1 0 0 A2 A1 A0 R/W 
            // U1A Value Regsiter = 0x12
            // U1A Direction Regsiter = 0x00
            // U1A Pullup Regsiter = 0x0C
            // U1B Value Regsiter = 0x13
            // U1B Direction Regsiter = 0x01
            // U1B Pullup Regsiter = 0x0D


            //*******************************************
            //TEST U1 with loopback test
            //*******************************************

            // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // Set the direction Register for U1 Port A
            // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

            outBuffer[0] = (byte)port;
            outBuffer[1] = (byte)0x91; //Lower CS for U1 
            udpClient.Send(outBuffer, 2, EtherIPEndPoint);

            outBuffer[0] = (byte)'S';
            outBuffer[1] = (byte)port;
            outBuffer[2] = (byte)0x03;
            outBuffer[3] = (byte)0x42; //Send the Control Byte for U1 (0x42)
            outBuffer[4] = (byte)0x00; //Send the Direction Register for Port A(0x00)
            outBuffer[5] = (byte)0x00; //Send the data to be loaded into Direction Register(0x00)

            udpClient.Send(outBuffer, 6, EtherIPEndPoint);

            // Read 6 bytes back from stream
            inBuffer = udpClient.Receive(ref EtherIPEndPoint);
           
            outBuffer[0] = (byte)port;
            outBuffer[1] = (byte)0x99; //Idle CS for U1 and U2

            udpClient.Send(outBuffer, 2, EtherIPEndPoint);

            // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // Set the direction Register for U1 Port B
            // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

            outBuffer[0] = (byte)port;
            outBuffer[1] = (byte)0x91; //Lower CS for U1 
            udpClient.Send(outBuffer, 2, EtherIPEndPoint);

            outBuffer[0] = (byte)'S';
            outBuffer[1] = (byte)port;
            outBuffer[2] = (byte)0x03;
            outBuffer[3] = (byte)0x42; //Send the Control Byte for U1 (0x42)
            outBuffer[4] = (byte)0x01; //Send the Direction Register for Port B (0x01)
            outBuffer[5] = (byte)0xFF; //Send the data to be loaded into Direction Register (0xFF)

            udpClient.Send(outBuffer, 6, EtherIPEndPoint);

            // Read 6 bytes back from stream
            inBuffer = udpClient.Receive(ref EtherIPEndPoint);
            
            outBuffer[0] = (byte)port;
            outBuffer[1] = (byte)0x99; //Idle CS for U1 and U2

           

            udpClient.Send(outBuffer, 2, EtherIPEndPoint);

            // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // Set Value on Port A U1 to 0xAA
            // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

            outBuffer[0] = (byte)port;
            outBuffer[1] = (byte)0x91; //Lower CS for U1
            udpClient.Send(outBuffer, 2, EtherIPEndPoint);

            outBuffer[0] = (byte)'S';
            outBuffer[1] = (byte)port;
            outBuffer[2] = (byte)0x03;
            outBuffer[3] = (byte)0x42; //Send the Control Byte for U1 (0x42)
            outBuffer[4] = (byte)0x12; //Send the Value Register for Port A (0x12)
            outBuffer[5] = (byte)0xAA; //Send the data to be loaded into Value Register (0xAA)

            udpClient.Send(outBuffer, 6, EtherIPEndPoint);

            // Read 6 bytes back from stream
            inBuffer = udpClient.Receive(ref EtherIPEndPoint);
         
            outBuffer[0] = (byte)port;
            outBuffer[1] = (byte)0x99; //Idle CS for U1 and U2
            udpClient.Send(outBuffer, 2, EtherIPEndPoint);

           

            TestTextBoxUDP.AppendText("U1 Port A 170 (0xAA) " + "\r\n");

            // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            // Read Value on Port B U1 should be 0xAA
            // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

            outBuffer[0] = (byte)port;
            outBuffer[1] = (byte)0x91; //Lower CS for U1 
            udpClient.Send(outBuffer, 2, EtherIPEndPoint);

            outBuffer[0] = (byte)'S';
            outBuffer[1] = (byte)port;
            outBuffer[2] = (byte)0x03;
            outBuffer[3] = (byte)0x43; //Send the Control Byte for U1 (0x43) as we want to read the register and not write
            outBuffer[4] = (byte)0x13; //Send the Value Register for Port B (0x13)
            outBuffer[5] = (byte)0x00; //Send dummy data for SPI read cycle

            udpClient.Send(outBuffer, 6, EtherIPEndPoint);

            // Read 6 bytes back from stream
            inBuffer = udpClient.Receive(ref EtherIPEndPoint);
          
            outBuffer[0] = (byte)port;
            outBuffer[1] = (byte)0x99; //Idle CS for U1 and U2
            udpClient.Send(outBuffer, 2, EtherIPEndPoint);

            

            byte portB = inBuffer[5];


            if (portB == 170)
            {
                TestTextBoxUDP.AppendText("Read U1 Port B " + portB + "\r\n");
                TestTextBoxUDP.AppendText("\r\n" + "Loopback Passed " + "\r\n");
                TestTextBoxUDP.AppendText("\r\n");
            }
            else
            {
                TestTextBoxUDP.AppendText("Read U1 Port B " + portB + "\r\n");
                TestTextBoxUDP.AppendText("\r\n" + "Loopback Failed " + "\r\n");
                TestTextBoxUDP.AppendText("\r\n");
               
            }

        }

    }

}
